from piscat.GUI.InputOutput import import_im2vid, import_RAW, video_cropping
from piscat.GUI.Visualization.fun_display_localization import Visulization_localization
from piscat.InputOutput import reading_videos, image_to_video, read_status_line

from PySide2 import QtCore
from PySide2 import QtWidgets

import os
import numpy as np
import cv2


class Reading(QtWidgets.QMainWindow):

    update_output = QtCore.Signal(object)

    def __init__(self):
        super(Reading, self).__init__()
        self.original_video = None
        self.filename = None
        self.folder = None

    def askdir_file(self):
        self.filename = False
        dialog = QtWidgets.QFileDialog(self)
        dialog.setWindowTitle('Open File')
        dialog.setFileMode(QtWidgets.QFileDialog.ExistingFile)
        if dialog.exec_() == QtWidgets.QDialog.Accepted:
            self.filename = dialog.selectedFiles()[0]
        if type(self.filename) is str:

            file_extention = os.path.splitext(self.filename)[1]
            if file_extention == ".pf" or file_extention == ".raw" or file_extention == ".bin" or file_extention == ".PF" or file_extention == ".RAW" or file_extention == ".BIN":
                return "Raw"
            elif file_extention == ".jpg" or file_extention == ".JPG" or file_extention == ".png" or file_extention == ".PNG":
                return "PNG"
            elif file_extention == ".tiff" or file_extention == ".TIFF":
                return "TIF"
            elif file_extention == ".avi" or file_extention == ".AVI":
                return "AVI"
            elif file_extention == ".TIF" or file_extention == ".tif":
                return "TIF"
            elif file_extention == ".py" or file_extention == ".PY":
                return "Python"

    def askdir(self):
        folder = QtWidgets.QFileDialog.getExistingDirectory(None, 'Select a folder:', '',
                                                                 QtWidgets.QFileDialog.ShowDirsOnly)
        return folder

    def read_video(self):
        title = self.askdir_file()
        if self.filename:
            if title == "Raw":
                self.info_image = import_RAW.RawImage(self.filename)
                while self.info_image.raw_data_update_flag:
                    QtWidgets.QApplication.processEvents()
                try:
                    video = reading_videos.read_binary(self.filename, img_width=self.info_image.width_size,
                                                       img_height=self.info_image.height_size,
                                                       image_type=self.info_image.set_bit_order + self.info_image.type)

                    if self.info_image.groupBox_cropping.isChecked():

                        self.original_video = video[
                                            self.info_image.frame_s:self.info_image.frame_e:self.info_image.frame_jump,
                                            self.info_image.width_size_s:self.info_image.width_size_e,
                                            self.info_image.height_size_s:self.info_image.height_size_e]

                    else:
                        self.original_video = video

                    self.original_video, status_line_info = self.status_line_remove(self.original_video)

                    if self.info_image.flag_display is True:
                        self.visualization_ = Visulization_localization()
                        self.visualization_.new_display(self.original_video, self.original_video, object=None, title='Raw video')

                    self.update_output.emit([self.original_video, title, self.filename, status_line_info])
                except:
                    self.msg_box = QtWidgets.QMessageBox()
                    self.msg_box.setWindowTitle("Warning!")
                    self.msg_box.setText("Selected parameters are not correct!")
                    self.msg_box.exec_()

            elif title == "PNG":
                png_img = reading_videos.read_png(self.filename)
                self.original_video = png_img

                self.visualization_ = Visulization_localization(self.filename)
                self.visualization_.new_display(self.original_video, self.original_video, object=None, title='PNG')

                self.update_output.emit([self.original_video, title, self.filename, None])

            elif title == "AVI":
                avi_video = reading_videos.read_avi(self.filename)

                self.info_image = video_cropping.Cropping(self)

                while self.info_image.raw_data_update_flag:
                    QtWidgets.QApplication.processEvents()

                if self.info_image.frame_e is not None:

                    if self.info_image.frame_e != -1:
                        self.original_video = avi_video[
                                              self.info_image.frame_s:self.info_image.frame_e:self.info_image.frame_jump,
                                              self.info_image.width_size_s:self.info_image.width_size_e,
                                              self.info_image.height_size_s:self.info_image.height_size_e]
                    elif self.info_image.frame_e == -1:
                        self.original_video = avi_video[self.info_image.frame_s:-1:self.info_image.frame_jump,
                                              self.info_image.width_size_s:self.info_image.width_size_e,
                                              self.info_image.height_size_s:self.info_image.height_size_e]
                else:
                    self.original_video = avi_video

                self.original_video = self.original_video.copy(order='C')

                if self.info_image.flag_display is True:
                    self.visualization_ = Visulization_localization()
                    self.visualization_.new_display(self.original_video, self.original_video, object=None, title='AVI')

                self.update_output.emit([self.original_video, title, self.filename, None])

            elif title == "TIF":
                tif_video = reading_videos.read_tif(self.filename)
                self.info_image = video_cropping.Cropping(self)

                while self.info_image.raw_data_update_flag:
                    QtWidgets.QApplication.processEvents()

                if self.info_image.flag_RGB2GRAY:
                    if tif_video.shape[2] == 4:
                        tif_video = cv2.cvtColor(tif_video, cv2.COLOR_BGR2GRAY)
                        tif_video = np.expand_dims(tif_video, axis=0)

                if self.info_image.frame_e is not None:

                    if self.info_image.frame_e != -1:
                        self.original_video = tif_video[
                                              self.info_image.frame_s:self.info_image.frame_e:self.info_image.frame_jump,
                                              self.info_image.width_size_s:self.info_image.width_size_e,
                                              self.info_image.height_size_s:self.info_image.height_size_e]
                    elif self.info_image.frame_e == -1:
                        self.original_video = tif_video[self.info_image.frame_s::self.info_image.frame_jump,
                                              self.info_image.width_size_s:self.info_image.width_size_e,
                                              self.info_image.height_size_s:self.info_image.height_size_e]
                else:
                    self.original_video = tif_video

                self.original_video = self.original_video.copy(order='C')

                if self.info_image.flag_display is True:
                    self.visualization_ = Visulization_localization()
                    self.visualization_.new_display(self.original_video, self.original_video, object=None, title='TIF')

                self.update_output.emit([self.original_video, title, self.filename, None])

            else:
                self.msg_box = QtWidgets.QMessageBox()
                self.msg_box.setWindowTitle("Warning!")
                self.msg_box.setText("The video bin_type is not defined!")
                self.msg_box.exec_()

        elif self.filename is False:
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("Filename is not valid.!")
            self.msg_box.exec_()

    def im2video(self):

        title = 'image2video'
        folder = self.askdir()
        self.info_image = import_im2vid.Image2Video(self)
        while self.info_image.raw_data_update_flag:
            QtWidgets.QApplication.processEvents()
        try:

            if self.info_image.im_type is not None:

                im2vid = image_to_video.Image2Video(path=folder, file_format=self.info_image.im_type,
                                                    width_size=self.info_image.width_size,
                                                    height_size=self.info_image.height_size,
                                                    image_type=np.dtype(self.info_image.set_bit_order + self.info_image.type),
                                                    reader_type=self.info_image.video_reader_type)
                video = im2vid()

                video, status_line_info = self.status_line_remove(video)

                self.original_video = video

                if self.info_image.flag_display is True:
                    self.visualization_ = Visulization_localization()
                    self.visualization_.new_display(self.original_video, self.original_video, object=None, title='im2video')

                self.update_output.emit([self.original_video, title, self.folder, status_line_info])
            else:
                self.msg_box = QtWidgets.QMessageBox()
                self.msg_box.setWindowTitle("Warning!")
                self.msg_box.setText("Type of image is not defined!")
                self.msg_box.exec_()
        except:
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("Selected parameters are not correct!")
            self.msg_box.exec_()

    def run_py_script(self):
        title = self.askdir_file()

        if title == "Python":
            self.threadpool.start(self.run_py_script_thread(self.filename))
            self.set_plain_text(" Done")

        else:
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("File bin_type is not valid.!")
            self.msg_box.exec_()

    def status_line_remove(self, video):
        statusLine = read_status_line.StatusLine(video)
        cut_frames, axis_status_line = statusLine.find_status_line()

        if axis_status_line == 'row' or axis_status_line == 'column':
            self.msg_box = QtWidgets.QMessageBox()
            self.msg_box.setWindowTitle("Warning!")
            self.msg_box.setText("status line remove" + "\nold shape:" + str(video.shape) + "\nnew shape:" + str(cut_frames.shape))
            self.msg_box.exec_()

        return cut_frames, axis_status_line

    def closeEvent(self, event):
        QtCore.QCoreApplication.instance().quit()
        print("closing PlaySetting")






from PyQt5 import QtCore, QtWidgets

from pyqtgraph import PlotWidget, plot
import pyqtgraph as pg

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure


class MplCanvas(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        super(MplCanvas, self).__init__(fig)


class UpdatingPlots(QtWidgets.QMainWindow):

    def __init__(self, *args, **kwargs):
        super(UpdatingPlots, self).__init__(*args, **kwargs)

        self.canvas = MplCanvas(self, width=6, height=6, dpi=100)
        self.setCentralWidget(self.canvas)

    def update_plot(self, ydata):
        self.canvas.axes.clear()  # Clear the canvas.
        self.canvas.axes.plot(ydata, 'r.')
        self.canvas.axes.ticklabel_format(style='sci', axis='y', scilimits=(0, 0))
        self.canvas.axes.grid()
        self.canvas.axes.set_ylabel('Pixel intensity')
        self.canvas.draw()


class UpdatingPlotsPyqtGraph(QtWidgets.QWidget):

    def __init__(self):
        super(UpdatingPlotsPyqtGraph, self).__init__()
        self.layout = QtWidgets.QVBoxLayout(self)

        self.graphWidget = pg.PlotWidget()
        self.graphWidget.setBackground('w')
        self.graphWidget.showGrid(x=True, y=True)
        styles = {"color": "#f00", "font-size": "20px"}
        self.graphWidget.setLabel("left", "Pixel intensity", **styles)

        self.layout.addWidget(self.graphWidget)

        pen = pg.mkPen(color=(255, 102, 0))
        self.data_line = self.graphWidget.plot([1], [1], pen=pen)

    def closeEvent(self, event):
        # QtCore.QCoreApplication.instance().quit()
        print("closing plot")



import pickle
import functools
import os
import inspect


def dump_array(file_name, arr):
    if os.path.exists(file_name):
        raise Exception('File already exists!')

    with open(file_name, 'wb') as f:
        pickle.dump(arr, f)



import numpy as np
import time
from piscat.BackgroundCorrection import NoiseFloor, DifferentialRollingAverage
from piscat.Preproccessing import Normalization
from piscat.InputOutput import StatusLine
from piscat.Localization import PSFsExtraction

from ..Fixtures.shared import *


def test_PSFsExtraction_non_jupyter(control_video):
    status_ = StatusLine(control_video)  # Reading the status line
    video_remove_status, status_information = status_.find_status_line()  # Examining the status line & removing it

    video_pn, _ = Normalization(video=video_remove_status).power_normalized()
    DRA_PN_cpFPNc = DifferentialRollingAverage(video=video_pn, batchSize=120)
    RVideo_PN_cpFPNc = DRA_PN_cpFPNc.differential_rolling(FFT_flag=False)
    PSF_l = PSFsExtraction(video=RVideo_PN_cpFPNc)
    PSFs = PSF_l.psf_detection_preview(function='dog',
                                min_sigma=1.6, max_sigma=1.7, sigma_ratio=1.1, threshold=5e-5,
                                overlap=0, mode='BOTH', frame_number=[1000], IntSlider_width='400px',
                                       title='Localization threshold on cpFPNc')

    assert len(PSFs) == 15



#
# import numpy as np
# import time
# from piscat.BackgroundCorrection import NoiseFloor
# from piscat.Preproccessing import Normalization
# from piscat.InputOutput import StatusLine
#
# from ..Fixtures.shared import *
#
# def _test_noisefloor_serial_vs_parallel(control_video):
#     status_ = StatusLine(control_video)
#     video_remove_status, status_information = status_.find_status_line()
#
#     video_pn, _ = Normalization(video=video_remove_status).power_normalized()
#     l_range = list(range(30, 150, 30))
#     start = time.time()
#     noise_floor_DRA_pn_parallel = NoiseFloor(video_pn, list_range=l_range, inter_flag_parallel_active=True)
#     stop = time.time()
#     dt_parallel = stop - start
#     start = time.time()
#     noise_floor_DRA_pn_serial = NoiseFloor(video_pn, list_range=l_range, inter_flag_parallel_active=False)
#     stop = time.time()
#     dt_serial = stop - start
#
#     assert dt_serial > dt_parallel
#
#     assert (noise_floor_DRA_pn_serial.mean >= 0).all()
#     np.testing.assert_array_almost_equal(noise_floor_DRA_pn_parallel.mean, noise_floor_DRA_pn_serial.mean)
#
#
# def test_noisefloor_results(control_video):
#     status_ = StatusLine(control_video)
#     video_remove_status, status_information = status_.find_status_line()
#
#     video_pn, _ = Normalization(video=video_remove_status).power_normalized()
#     l_range = list(range(90, 150, 30))
#     noise_floor_DRA_pn = NoiseFloor(video_pn, list_range=l_range, inter_flag_parallel_active=False)
#     assert np.nanargmin(noise_floor_DRA_pn.mean) == 1
#     assert np.min(noise_floor_DRA_pn.mean) == pytest.approx(2.7E-4, rel=0.1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import exists, dirname, realpath
from setuptools import setup, find_packages
import sys

author = u"PiSCAT developers"
# authors in alphabetical order
authors = [
    "Houman Mirzaalian",
    "Reza Gholami",
]
description = 'Interferometric scattering microscopy with Python'
name = 'piscat'
year = "2021"


sys.path.insert(0, realpath(dirname(__file__))+"/"+name)
try:
    from piscat import version
except BaseException:
    version = "unknown"

setup(
    name=name,
    author=author,
    author_email='houman.mirzaalian-dastjerdi@mpl.mpg.de',
    url='https://github.com/SandoghdarLab/PiSCAT',
    version=version,
    packages=find_packages(),
    package_dir={name: name},
    include_package_data=True,
    license="GPL v3",
    description=description,
    long_description=open('README.md').read() if exists('README.md') else '',
    long_description_content_type="text/markdown",
    install_requires=[
                        "numpy>=1.19.0",
                        "scipy>=0.14.0",
                        "matplotlib",
                        "opencv-python",
                        "pandas",
                        "PyQt5",
                        "Pyside2",
                        "pyqtgraph",
                        "PyQtWebEngine",
                        "scikit_learn",
                        "scikit-image",
                        "tqdm",
                        "trackpy",
                        "joblib",
                        "psutil",
                        "ipywidgets",
                        "networkx",
                        "numba",
                        "requests",
                        "wget",
                        "versioned-hdf5",
                        "h5py",
                        "imageio-ffmpeg"
                      ],
    # not to be confused with definitions in pyproject.toml [build-system]
    setup_requires=["pytest-runner"],
    python_requires=">=3.8",
    tests_require=["pytest", "pytest-mock"],
    keywords=[],
    classifiers=['Operating System :: OS Independent',
                 'Programming Language :: Python :: 3',
                 'Topic :: Scientific/Engineering :: Visualization',
                 'Intended Audience :: Science/Research',
                 ],
    platforms=['ALL'],
)


import numpy as np
import math
import pandas as pd

from piscat.InputOutput.cpu_configurations import CPUConfigurations
from tqdm.autonotebook import tqdm


class SpatialFilter():

    def __init__(self):
        """
        We have a `SpatialFilter` class in PiSCAT that allows users to filter `outlier_frames`
        that have a strong vibration or a particle flying by, `dense_PSFs`, and non-symmetric PSFs that
        may not properly resemble the iPSF expected from the experimental setup.
        The threshold parameter in each of these filters determines the filter's sensitivity.
        """
        self.cpu = CPUConfigurations()

    def list_frames(self, df_PSFs):
        list_frames = df_PSFs['frame'].tolist()
        list_frames = np.sort(np.unique(list_frames))
        list_frames = list_frames.tolist()
        return list_frames

    def outlier_frames(self, df_PSFs, threshold=20):
        """
        This function eliminates all detected PSFs in the frame that are greater than the threshold value.
        PSFs that were detected in unstable frames are reduced using this method.

        Parameters
        ----------
        df_PSFs: pandas dataframe
            The data frame contains PSFs locations( x, y, frame, sigma)

        threshold: int
            Maximum number of PSFs in one frame.

        Returns
        -------
        filter_df_PSFs: pandas dataframe
            The filter data frame contains PSFs locations( x, y, frame, sigma)
        """
        if df_PSFs.shape[0] == 0 or df_PSFs is None:
            raise ValueError('---data frames is empty!---')

        print("\nstart removing crappy frames --->", end=" ")
        his_all_particles = df_PSFs['frame'].value_counts()
        temp = his_all_particles.where(his_all_particles <= threshold)
        select_particles = temp[~temp.isnull()]
        index_particles = select_particles.index
        filter_df_PSFs = df_PSFs.loc[df_PSFs['frame'].isin(index_particles)]
        print('Done!')
        return filter_df_PSFs

    def dense_PSFs(self, df_PSFs, threshold=0):
        """
        Parameters
        ----------
        df_PSFs: pandas dataframe
            The data frame contains PSFs locations( x, y, frame, sigma)

        threshold: float
            It specifies the portion of the overlay that two PSFs must have to remove from the list.

        Returns
        -------
        filter_df_PSFs: pandas dataframe
                    The filter data frame contains PSFs locations( x, y, frame, sigma)
        """
        if df_PSFs.shape[0] == 0 or df_PSFs is None:
            print('---data frames is empty!---')
            return df_PSFs

        if type(df_PSFs) is pd.core.frame.DataFrame:
            df_PSFs = df_PSFs
        else:
            raise ValueError('Input does not have correct type! This function needs panda data frames.')

        list_frames = self.list_frames(df_PSFs)

        self.particles_after_closeFilter = df_PSFs
        self.index_particles_filter = self.particles_after_closeFilter.index

        num_particles = self.particles_after_closeFilter.shape[0]

        print('\n---Cleaning the df_PSFs that have drift without parallel loop---')
        self.point_1 = np.zeros((1, 2), dtype=np.float64)
        self.point_2 = np.zeros((1, 2), dtype=np.float64)
        self.remove_list_close = []

        for frame_num in tqdm(list_frames):
            particle = self.particles_after_closeFilter.loc[self.particles_after_closeFilter['frame'] == frame_num]
            index_list = [index for index in particle.index]
            particle_X = particle['x'].tolist()
            particle_Y = particle['y'].tolist()
            particle_sigma = particle['sigma'].tolist()

            if len(index_list) != 1:
                for i_ in range(len(particle_X)):
                    self.point_1[0, 0] = particle_X[i_]
                    self.point_1[0, 1] = particle_Y[i_]
                    sigma_1 = particle_sigma[i_]

                    count_ = i_ + 1
                    while count_ <= (len(particle_X) - 1):

                        self.point_2[0, 0] = particle_X[count_]
                        self.point_2[0, 1] = particle_Y[count_]
                        sigma_2 = particle_sigma[count_]

                        distance = math.sqrt(((self.point_1[0, 0] - self.point_2[0, 0]) ** 2) + (
                                (self.point_1[0, 1] - self.point_2[0, 1]) ** 2))
                        tmp = (math.sqrt(2) * (sigma_1 + sigma_2))
                        if distance <= ((math.sqrt(2) * (sigma_1 + sigma_2)) - (threshold * tmp)):
                            self.remove_list_close.append(index_list[i_])
                            self.remove_list_close.append(index_list[count_])

                        count_ = count_ + 1

        remove_list = list(set(self.remove_list_close))
        self.particles_after_closeFilter = self.particles_after_closeFilter.drop(remove_list, axis=0, errors='ignore')
        print("\nNumber of PSFs before filters = {}".format(num_particles))
        print("\nNumber of PSFs after filters = {}".format(self.particles_after_closeFilter.shape[0]))

        return self.particles_after_closeFilter.reset_index(drop=True)

    def symmetric_PSFs(self, df_PSFs, threshold=0.7):
        """
        Remove astigmatism-affected PSFs with this filter.

        Parameters
        ----------
        df_PSFs: pandas dataframe
            The data frame contains PSFs locations( x, y, frame, sigma)

        threshold: float
            The smallest sigma ratio that is acceptable (sigma max/sigma min).

        Returns
        -------
        df_PSF_thr: pandas dataframe
            The filter data frame contains PSFs locations( x, y, frame, sigma)

        """

        if df_PSFs is not None and df_PSFs.shape[0] > 0:
            if 'Sigma_ratio' in df_PSFs.keys():
                idx_thr_sigma = df_PSFs.Sigma_ratio > threshold
                idx_nan_sigma = df_PSFs.Sigma_ratio.isnull()

                df_PSF_thr = df_PSFs[idx_thr_sigma.tolist() or idx_nan_sigma.tolist()]

                return df_PSF_thr

            else:
                raise Exception('---Sigma ratio does not exist in data frames---')

        else:
            raise Exception('---data frames is empty!---')

    def remove_side_lobes_artifact(self, df_PSFs, threshold=0):
        """
        This filter removes false detections on side lobes of PSFs caused by the localization algorithm by comparing center intensity contrast.

        Parameters
        ----------
        df_PSFs: pandas dataframe
            The data frame contains PSFs locations( x, y, frame, sigma, center_intensity)

        threshold: float
            It specifies the portion of the overlay that two PSFs must have to remove from the list.

        Returns
        -------
        filter_df_PSFs: pandas dataframe
            The filter data frame contains PSFs locations( x, y, frame, sigma, center_intensity)
        """
        if df_PSFs.shape[0] == 0 or df_PSFs is None:
            print('---data frames is empty!---')
            return df_PSFs

        if type(df_PSFs) is pd.core.frame.DataFrame:
            df_PSFs = df_PSFs
        else:
            raise ValueError('Input does not have correct type! This function needs panda data frames.')

        list_frames = self.list_frames(df_PSFs)

        self.particles_after_closeFilter = df_PSFs
        self.index_particles_filter = self.particles_after_closeFilter.index

        num_particles = self.particles_after_closeFilter.shape[0]

        print('\n---Cleaning the df_PSFs that have side lobs without parallel loop---')
        self.point_1 = np.zeros((1, 2), dtype=np.float64)
        self.point_2 = np.zeros((1, 2), dtype=np.float64)
        self.remove_list_close = []

        for frame_num in tqdm(list_frames):
            particle = self.particles_after_closeFilter.loc[self.particles_after_closeFilter['frame'] == frame_num]
            index_list = [index for index in particle.index]
            particle_X = particle['x'].tolist()
            particle_Y = particle['y'].tolist()
            particle_sigma = particle['sigma'].tolist()
            particle_center_intensity = particle['center_intensity'].tolist()

            if len(index_list) != 1:
                for i_ in range(len(particle_X)):
                    self.point_1[0, 0] = particle_X[i_]
                    self.point_1[0, 1] = particle_Y[i_]
                    sigma_1 = particle_sigma[i_]

                    count_ = i_ + 1
                    while count_ <= (len(particle_X) - 1):

                        self.point_2[0, 0] = particle_X[count_]
                        self.point_2[0, 1] = particle_Y[count_]
                        sigma_2 = particle_sigma[count_]

                        distance = math.sqrt(((self.point_1[0, 0] - self.point_2[0, 0]) ** 2) + (
                                (self.point_1[0, 1] - self.point_2[0, 1]) ** 2))
                        tmp = (math.sqrt(2) * (sigma_1 + sigma_2))
                        if distance <= ((math.sqrt(2) * (sigma_1 + sigma_2)) - (threshold * tmp)):

                            intesity_1 = particle_center_intensity[i_]
                            intesity_2 = particle_center_intensity[count_]

                            if np.abs(intesity_1) == np.abs(intesity_2):
                                self.remove_list_close.append(index_list[i_])
                                self.remove_list_close.append(index_list[count_])
                            if np.abs(intesity_1) > np.abs(intesity_2):
                                self.remove_list_close.append(index_list[count_])
                            if intesity_1 < intesity_2:
                                self.remove_list_close.append(index_list[i_])
                        count_ = count_ + 1

        remove_list = list(set(self.remove_list_close))
        self.particles_after_closeFilter = self.particles_after_closeFilter.drop(remove_list, axis=0, errors='ignore')
        print("\nNumber of PSFs before filters = {}".format(num_particles))
        print("\nNumber of PSFs after filters = {}".format(self.particles_after_closeFilter.shape[0]))

        return self.particles_after_closeFilter.reset_index(drop=True)







# from PySide2.QtCore import *
# from PySide2.QtWidgets import *
# from PySide2.QtGui import *
# from PySide2.QtWebEngineWidgets import *

from PySide2.QtCore import QUrl
from PySide2.QtWidgets import QMainWindow, QApplication
from PySide2.QtWebEngineWidgets import QWebEngineView

import os
import sys


class Help(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(Help, self).__init__(*args, **kwargs)

        self.browser = QWebEngineView()
        self.browser.setUrl(QUrl("https://piscat.readthedocs.io/"))
        self.setCentralWidget(self.browser)
        self.show()


# class Help(QMainWindow):
#
#     def __init__(self, *args, **kwargs):
#         super(Help, self).__init__(*args, **kwargs)
#
#         # url = QUrl('https://piscat.readthedocs.io/')
#         # if not QDesktopServices.openUrl(url):
#         #     QMessageBox.warning(self, 'Open Url', 'Could not open url')
#
#         self.browser = QWebEngineView()
#         self.browser.setUrl(QUrl("https://piscat.readthedocs.io/"))
#
#         self.setCentralWidget(self.browser)
#
#         self.show()



from __future__ import print_function

import matplotlib.pyplot as plt
import numpy as np


def plot3(X, Y, Z, scale='(um)', title=''):
    """
    3D localization plotting

    Parameters
    ----------
    X: list
        X-position of PSFs.

    Y: list
        Y-position of PSFs.

    Z: list
        Z-position of PSFs.

    scale: str
        Measurement scale. e.g ('nm')

    title: str
        The title of the figure.

    """
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    ax.scatter(X, Y, Z)

    ax.set_xlabel('X' + scale)
    ax.set_ylabel('Y' + scale)
    ax.set_zlabel('Z' + scale)

    plt.title(title)
    plt.show()


def plot2df(df_PSFs, pixel_size=1, scale='(um)', title='', flag_label=False):
    """
    2D localization plotting with color code for each particle.

    Parameters
    ----------
    df_PSFs: panda data frame
        Data frame contains PSFs localization and ID.

    pixel_size:
        The camera's pixel size is used to scale the results of the localization.

    scale: str
        Measurement scale. e.g ('nm')

    title: str
        The title of the figure.

    flag_label: bool


    """
    groups = df_PSFs.groupby('particle')

    fig, ax = plt.subplots()
    for name, group in groups:
        ax.plot(group.x*pixel_size, group.y*pixel_size, marker='.', linestyle='', ms=3, label=name)
    if flag_label:
        ax.legend()
    ax.set_xlabel(scale)
    ax.set_ylabel(scale)
    plt.title(title)
    plt.show()


def plot_histogram(data, title, fc='C1', ec='k'):
    """
    Plotting a histogram out of the data.

    Parameters
    ----------
    data: list
        Input data.

    title: str
        The title of the figure.

    fc: str
        Face color of histogram.

    ec: str
        Edge color of histogram.

    """

    plt.figure()
    plt.hist(data, alpha=0.7, bins=None, density=False, stacked=True, fc=fc, ec=ec)
    plt.title(title + '\nMedian: ' + str(np.median(data)))
    plt.ylabel('#Counts', fontsize=18)
    plt.xlabel('Length of linking', fontsize=18)
    plt.tight_layout()
    plt.show()

